#include<stdio.h>

#define NUMBER '*'
#define FILLED '#'
#define EMPTY '_'
#define BLACK -1
#define BLANK 0
#define OUT_OF_GRID -1

void makebox(FILE *fp, float drawing_size, int dim, int *grid);

main(int argc, char **argv)
{
   FILE *inp,*outp;
   int i,j,k;
   char c,name[50];
   int *grid, *clues_across, *clues_down;
   int pos,dim, index, index_across, index_down;
   int pos_left, pos_right, pos_top, pos_bottom;
   float box_size;
   float DRAW_SIZE=18;

   /* Open input and output files. */

   if(argc < 2) {
      inp=stdin;
      outp=stdout;
   }
   else if(argc >2) {
      inp=fopen(argv[1],"r");
      outp=fopen(argv[2],"w");
   }
   else {
      inp=fopen(argv[1],"r");
      i=0;
      j=0;
      while((c=argv[1][i++])!='\0') {
         if(c=='/') j=0;
         else if(c=='.') break;
         else name[j++]=c;
      }
      name[j]='\0';
      strcat(name,".tex");
      outp=fopen(name,"w");
   }

   /* Read the grid structure */

   fscanf(inp,"%d\n",&dim);
   grid=(int *)malloc(dim*dim*sizeof(int));

   /*Once and for all: 

      |
   ^  |
   j  |
      |
      |_________
	i >
   */

   for(j=dim-1;j>=0;j--) {
      for(i=0;i<dim;i++) {
	 pos=(i*dim)+j;
	 fscanf(inp,"%c",&c);
	 grid[pos]=c;
      }
      fscanf(inp,"\n");
   }

   for(j=dim-1;j>=0;j--) {
      for(i=0;i<dim;i++) {
	 pos=(i*dim)+j;
	 if(grid[pos]!=EMPTY)
	    continue;
	 if(i==0)
	    pos_left=OUT_OF_GRID;
	 else
	    pos_left=((i-1)*dim)+j;
	 if(i==dim-1)
	    pos_right=OUT_OF_GRID;
	 else
	    pos_right=((i+1)*dim)+j;
	 if(j==dim-1)
	    pos_top=OUT_OF_GRID;
	 else
	    pos_top=(i*dim)+j+1;
	 if(j==0)
	    pos_bottom=OUT_OF_GRID;
	 else
	    pos_bottom=(i*dim)+j-1;

         if(pos_right != OUT_OF_GRID) {
	    if(grid[pos_right] == EMPTY) {
	       if(pos_left==OUT_OF_GRID)
		  grid[pos]=NUMBER;
	       else if(grid[pos_left]==FILLED)
		  grid[pos]=NUMBER;
	    }
	 }
         if(pos_bottom != OUT_OF_GRID) {
	    if(grid[pos_bottom] == EMPTY) {
	       if(pos_top==OUT_OF_GRID)
		  grid[pos]=NUMBER;
	       else if(grid[pos_top]==FILLED)
		  grid[pos]=NUMBER;
	    }
	 }
      }
   }

   index=0;
   for(j=dim-1;j>=0;j--) {
      for(i=0;i<dim;i++) {
	 pos=(i*dim)+j;
	 switch(grid[pos]) {
	    case FILLED:
	       grid[pos]=BLACK;
	       break;
	    case EMPTY:
	       grid[pos]=BLANK;
	       break;
	    default:
	       index++;
	       grid[pos]=index;
	       break;
	 }
      }
   }

   clues_across=(int *)malloc(dim*dim*sizeof(int));
   clues_down=(int *)malloc(dim*dim*sizeof(int));
   index_across=0;
   index_down=0;
   for(j=dim-1;j>=0;j--) {
      for(i=0;i<dim;i++) {
	 pos=(i*dim)+j;
	 if(grid[pos]<=BLANK)
	    continue;
	 if(i==0)
	    pos_left=OUT_OF_GRID;
	 else
	    pos_left=((i-1)*dim)+j;
	 if(i==dim-1)
	    pos_right=OUT_OF_GRID;
	 else
	    pos_right=((i+1)*dim)+j;
	 if(j==dim-1)
	    pos_top=OUT_OF_GRID;
	 else
	    pos_top=(i*dim)+j+1;
	 if(j==0)
	    pos_bottom=OUT_OF_GRID;
	 else
	    pos_bottom=(i*dim)+j-1;
		  
         if(pos_right != OUT_OF_GRID) {
	    if(grid[pos_right] >= BLANK) {
	       if(pos_left==OUT_OF_GRID)
		  clues_across[index_across++]=grid[pos];
	       else if(grid[pos_left]==BLACK)
		  clues_across[index_across++]=grid[pos];
	    }
	 }
         if(pos_bottom != OUT_OF_GRID) {
	    if(grid[pos_bottom] >= BLANK) {
	       if(pos_top==OUT_OF_GRID)
		  clues_down[index_down++]=grid[pos];
	       else if(grid[pos_top]==BLACK)
		  clues_down[index_down++]=grid[pos];
	    }
	 }
      }
   }

   printf("Across:\n");
   for(i=0;i<index_across;i++)
      printf("%3d ", clues_across[i]);
   printf("\n");
   printf("Down:\n");
   for(i=0;i<index_down;i++)
      printf("%3d ", clues_down[i]);
   printf("\n");

   fprintf(outp, "\\documentstyle[cross]{article}\n");
   fprintf(outp, "\\begin{document}\n");
   makebox(outp, DRAW_SIZE, dim, grid);
   fprintf(outp, "\\end{document}\n");
}

void makebox(FILE *fp, float drawing_size, int dim, int *grid)
{
   float box_size,total_size,xpos,ypos;
   float numberxpos,numberypos;
   int i,j,pos;

   box_size=drawing_size/(float)dim; 
   box_size=0.6;
   total_size=box_size*dim;
   if(total_size>drawing_size){
      box_size=drawing_size/(float)dim;
      total_size=box_size*dim;
   }

   /*
   fprintf(fp,"\\newcounter{gridnum}\n");
   fprintf(fp,"\\begin{picture}(%5.3f,%5.3f)\n",drawing_size,drawing_size);
   */
   fprintf(fp,"\\begin{picture}(%5.3f,%5.3f)\n",total_size,total_size);
   fprintf(fp,"\\multiput(0,0)(0,%5.3f){%d}{\\line(1,0){%5.3f}}\n",box_size,dim+1,total_size);
   fprintf(fp,"\\multiput(0,0)(%5.3f,0){%d}{\\line(0,1){%5.3f}}\n",box_size,dim+1,total_size);

   fprintf(fp,"\\linethickness{%5.3fcm}\n",box_size);
   for(j=dim-1;j>=0;j--) {
      for(i=0;i<dim;i++) {
	 xpos=(i+0.5)*box_size;
	 ypos=j*box_size;
	 numberxpos=(i+0.05)*box_size;
	 numberypos=(j-0.05)*box_size;
	 pos=i*dim+j;
	 switch(grid[pos]) {
	    case BLACK:
	       fprintf(fp,"\\put(%5.3f,%5.3f){\\line(0,1){%5.3f}}\n",xpos,ypos,box_size);
	       break;
	    case BLANK:
	       break;
	    default:
	       /*
               fprintf(fp, "\\addtocounter{gridnum}{1}\\put(%5.3f,%5.3f)",numberxpos,numberypos);
	       fprintf(fp,"{\\makebox(%5.3f,%5.3f)[lt]{\\arabic{gridnum}}}\n",box_size,box_size);
	       */
               fprintf(fp, "\\put(%5.3f,%5.3f)",numberxpos,numberypos);
	       fprintf(fp,"{\\makebox(%5.3f,%5.3f)[lt]{%d}}\n",box_size,box_size,grid[pos]);
	       break;
	 }
      }
   }
   fprintf(fp,"\\end{picture}\n");
}
